// Copyright © Advanced Micro Devices, Inc., or its affiliates.
// SPDX-License-Identifier: MIT

#pragma once

typedef enum
{
    HIPDNN_INVALID_TYPE = 0,
    HIPDNN_BACKEND_ENGINE_DESCRIPTOR,
    HIPDNN_BACKEND_ENGINECFG_DESCRIPTOR,
    HIPDNN_BACKEND_ENGINEHEUR_DESCRIPTOR,
    HIPDNN_BACKEND_EXECUTION_PLAN_DESCRIPTOR,
    HIPDNN_BACKEND_INTERMEDIATE_INFO_DESCRIPTOR,
    HIPDNN_BACKEND_KNOB_CHOICE_DESCRIPTOR,
    HIPDNN_BACKEND_KNOB_INFO_DESCRIPTOR,
    HIPDNN_BACKEND_LAYOUT_INFO_DESCRIPTOR,
    HIPDNN_BACKEND_OPERATION_GEN_STATS_DESCRIPTOR,
    HIPDNN_BACKEND_OPERATIONGRAPH_DESCRIPTOR,
    HIPDNN_BACKEND_VARIANT_PACK_DESCRIPTOR,
    HIPDNN_BACKEND_KERNEL_CACHE_DESCRIPTOR,
    HIPDNN_BACKEND_OPERATION_PAGED_CACHE_LOAD_DESCRIPTOR,

} hipdnnBackendDescriptorType_t;
