// automatically generated by the FlatBuffers compiler, do not modify


#ifndef FLATBUFFERS_GENERATED_CONVOLUTIONCOMMON_HIPDNN_SDK_DATA_OBJECTS_H_
#define FLATBUFFERS_GENERATED_CONVOLUTIONCOMMON_HIPDNN_SDK_DATA_OBJECTS_H_

#include "flatbuffers/flatbuffers.h"

// Ensure the included flatbuffers.h is the same version as when this file was
// generated, otherwise it may not be compatible.
static_assert(FLATBUFFERS_VERSION_MAJOR == 25 &&
              FLATBUFFERS_VERSION_MINOR == 12 &&
              FLATBUFFERS_VERSION_REVISION == 19,
             "Non-compatible flatbuffers version included");


namespace hipdnn_sdk {
namespace data_objects {

enum class ConvMode : int8_t {
  UNSET = 0,
  CONVOLUTION = 1,
  CROSS_CORRELATION = 2,
  MIN = UNSET,
  MAX = CROSS_CORRELATION
};

inline const ConvMode (&EnumValuesConvMode())[3] {
  static const ConvMode values[] = {
    ConvMode::UNSET,
    ConvMode::CONVOLUTION,
    ConvMode::CROSS_CORRELATION
  };
  return values;
}

inline const char * const *EnumNamesConvMode() {
  static const char * const names[4] = {
    "UNSET",
    "CONVOLUTION",
    "CROSS_CORRELATION",
    nullptr
  };
  return names;
}

inline const char *EnumNameConvMode(ConvMode e) {
  if (::flatbuffers::IsOutRange(e, ConvMode::UNSET, ConvMode::CROSS_CORRELATION)) return "";
  const size_t index = static_cast<size_t>(e);
  return EnumNamesConvMode()[index];
}

}  // namespace data_objects
}  // namespace hipdnn_sdk

#endif  // FLATBUFFERS_GENERATED_CONVOLUTIONCOMMON_HIPDNN_SDK_DATA_OBJECTS_H_
