// Copyright © Advanced Micro Devices, Inc., or its affiliates.
// SPDX-License-Identifier:  MIT

#pragma once

typedef enum
{
    HIPDNN_TYPE_HANDLE = 0,
    HIPDNN_TYPE_DATA_TYPE,
    HIPDNN_TYPE_BOOLEAN,
    HIPDNN_TYPE_INT64,
    HIPDNN_TYPE_FLOAT,
    HIPDNN_TYPE_DOUBLE,
    HIPDNN_TYPE_VOID_PTR,
    HIPDNN_TYPE_HEUR_MODE,
    HIPDNN_TYPE_KNOB_TYPE,
    HIPDNN_TYPE_NAN_PROPOGATION,
    HIPDNN_TYPE_NUMERICAL_NOTE,
    HIPDNN_TYPE_LAYOUT_TYPE,
    HIPDNN_TYPE_ATTRIB_NAME,
    HIPDNN_TYPE_BACKEND_DESCRIPTOR,
    HIPDNN_TYPE_GENSTATS_MODE,
    HIPDNN_TYPE_BN_FINALIZE_STATS_MODE,
    HIPDNN_TYPE_BEHAVIOR_NOTE,
    HIPDNN_TYPE_TENSOR_REORDERING_MODE,
    HIPDNN_TYPE_INT32,
    HIPDNN_TYPE_CHAR,
    HIPDNN_TYPE_SIGNAL_MODE,
    HIPDNN_TYPE_FRACTION,
    HIPDNN_TYPE_NORM_FWD_PHASE,
    HIPDNN_TYPE_RNG_DISTRIBUTION
} hipdnnBackendAttributeType_t;
